#!/usr/bin/python3

import rospy
import numpy as np
from sensor_msgs.msg import NavSatFix
from geometry_msgs.msg import PointStamped
from dji_osdk_ros.msg import VOPosition
from geometry_msgs.msg import Transform
from geometry_msgs.msg import Vector3
from geometry_msgs.msg import Quaternion
from scipy.spatial.transform import Rotation
from tools import lat_lon2meters, meters2lat_lon
import math

homeSetFlag = False
# homePoint = PointStamped()
homePoint = NavSatFix()
transform = Transform()
angleOffset = 0.0

def point_rotation(origin, pt, ang):
    # returns the pt rotated about the origin by ang (ang in degrees)
    c = math.cos(math.radians(ang))
    s = math.sin(math.radians(ang))
    # translate to origin
    pt_temp = [pt[0] - origin[0], pt[1] - origin[1]]
    pt_spun = [ pt_temp[0]*c - pt_temp[1]*s, pt_temp[0]*s + pt_temp[1]*c ]
    # translate back to frame
    pt_spun = [pt_spun[0] + origin[0], pt_spun[1] + origin[1]]
    return pt_spun



def callback(data):
    global homeSetFlag
    global homePoint
    global transform
    if not homeSetFlag: # mildly hacky, depends on position format we're using for feedback. might build into bus driver and pub from there
        # set up home point in yee pee es
        homePoint = data

        # homePoint.point.x = data.x
        # homePoint.point.y = data.y
        # homePoint.point.z = data.z
        # homePoint.header = data.header
        homeSetFlag = True
        rospy.loginfo("setting home point...")

    # convert and compare against home point
    translation = Vector3()
    rotation = Quaternion()

    # rotate the xy plane by angle
    # dataPointSpun = point_rotation([0, 0], [data.x, data.y], angleOffset)

    homePointMeters = lat_lon2meters([homePoint.latitude, homePoint.longitude])
    currentPointMeters = lat_lon2meters([data.latitude, data.longitude])

    currentOffset = [currentPointMeters[0] - homePointMeters[0], currentPointMeters[1] - homePointMeters[1]]


    # angleOffset = -33.0
    # rospy.loginfo(angleOffset)
    # rospy.loginfo(currentOffset)
    angleOffset = rospy.get_param("~angleOffset")
    currentRotatedOffset = point_rotation([0,0], currentOffset, angleOffset)
    # rospy.loginfo(currentRotatedOffset)

    translation.x = currentRotatedOffset[1]
    translation.y = currentRotatedOffset[0]
    translation.z = data.altitude - homePoint.altitude

    transform.translation = translation

    rot = Rotation.from_euler('xyz', [0, 0, 0], degrees = True) # be not rotated pls
    zeroQuaternion = rot.as_quat()
    rotation.x = zeroQuaternion[0]
    rotation.y = zeroQuaternion[1]
    rotation.z = zeroQuaternion[2]
    rotation.w = zeroQuaternion[3]

    transform.rotation = rotation

def main():
    global transform
    rospy.init_node("poseFeedbackConversion")
    rospy.Subscriber("dji_osdk_ros/gps_position", NavSatFix, callback, queue_size = 1)
    posePublisher = rospy.Publisher("pose_feedback", Transform, queue_size = 1)
    while not rospy.is_shutdown():
        posePublisher.publish(transform)


if __name__ == '__main__':
    main()
