#!/usr/bin/python3
from tools import meters2lat_lon, lat_lon2meters
import numpy as np
import rospy


def normalizeAngle(angle):
    newAngle = angle%360
    newAngle = (newAngle + 360)%360
    if(newAngle >= 180):
        newAngle -= 360
    return newAngle

def generateWaypoints():

    rospy.init_node("waypointGenerator")
    anchorPoint = rospy.get_param("~anchorPoint")
    waypointAltitude = rospy.get_param("~altitude")

    # ring sizes
    big = 10
    med = 5
    sml = 3

    apMeters = lat_lon2meters(anchorPoint)

    waypointOffsets = [
    [-big, -big],
    [big, -big],
    [big, big],
    [med, med],
    [-med, med],
    [-med, -med],
    [-sml, -sml],
    [sml, -sml],
    [sml, sml],
    [big, -big],
    ]
    tempDict = {"waypoints" : {}, "positions" : {}, "numWaypoints" : 0, "numPositions" : 0}
    waypointDict = {"robot_0" : tempDict, "robot_1" : tempDict}

    numWaypoints = len(waypointOffsets)
    numPositions = len(waypointOffsets) # TODO, add position list to parameters

    for i, offset in enumerate(waypointOffsets):
        waypointLatLon = meters2lat_lon([apMeters[0] + offset[0], apMeters[1] + offset[1]])
        waypointLatLonRev = meters2lat_lon([apMeters[0] - offset[0], apMeters[1] - offset[1]])
        waypointHeading = normalizeAngle(45*i)
        # waypoints
        waypointDict["robot_0"]["waypoints"].update({"waypoint_{}".format(i) : [i, waypointLatLon[0], waypointLatLon[1], waypointAltitude, waypointHeading]})
        waypointDict["robot_1"]["waypoints"].update({"waypoint_{}".format(i) : [i, waypointLatLonRev[0], waypointLatLonRev[1], waypointAltitude, waypointHeading]})
        # positions
        waypointDict["robot_0"]["positions"].update({"position_{}".format(i) : [i, offset[0], offset[1], waypointAltitude, waypointHeading]})
        waypointDict["robot_1"]["positions"].update({"position_{}".format(i) : [i, -offset[0], -offset[1], waypointAltitude, waypointHeading]})

    # do this betta
    waypointDict["robot_0"]["numWaypoints"] = numWaypoints
    waypointDict["robot_0"]["numPositions"] = numPositions
    waypointDict["robot_1"]["numWaypoints"] = numWaypoints
    waypointDict["robot_1"]["numPositions"] = numPositions

    rospy.set_param("/robot_list", waypointDict)

if __name__ == "__main__":
    generateWaypoints()
