# NVIDIA Jetson Xavier NX Setup

Below are the steps required to set up a stock Jetson to use this code base:

* Set up the SD card to have an OS on it, either by:
    - Using NVIDIA's set up proceedure found [here](https://developer.nvidia.com/embedded/learn/get-started-jetson-xavier-nx-devkit#prepare), which then would require a lot of installations/additional set up (OpenCV, Ros, etc.).
    - Using the CAS Lab's own images found `[TBD]`.

* If using the NVIDIA option a few more things need to be configured:
    - Follow the steps listed in the DJI Onboard SDK software set up page [here](https://developer.dji.com/onboard-sdk/documentation/development-workflow/environment-setup.html) (also can be helpful for debugging, a complete list of vehicle activation checks can be found [here](https://forum.dji.com/thread-216529-1-1.html))
    - The udev rules file is slightly different than what DJI suggests, this is because the USB device name needs to be static so it can be found every time. The udev file contents below is designed for use with [this](https://www.adafruit.com/product/954) USB device. To use a different device the `idVendor` field can be found by inspecting `dmesg` or `lsusb`

* The hosts and hostname will need to be configured regardless of the image source
    - The `/etc/hosts` template, listed below, makes it easier to ssh into the Jetson and launch ROS nodes.
    - The `/etc/hostname` file holds the 

### UDEV Rules File

The udev rules contents, to be placed in `/etc/udev/rules.d/DJIDevice.rules`, is listed below

`SUBSYSTEM=="usb", ATTRS{idVendor}=="2ca3", MODE="0666"
SUBSYSTEM=="tty", ATTRS{idVendor}=="10c4", ATTRS{idProduct}=="ea60", SYMLINK+="dji_usb"`

### Hosts File

The hosts contents, to be placed in `/etc/hosts` is listed below

`
127.0.0.1       localhost
192.168.1.100   pop-os
192.168.1.230   pop-os
127.0.0.1       yeats
192.168.1.101   neruda
192.168.1.191   nerudaHouse
192.168.1.102   plath
192.168.1.24    plathHouse
192.168.1.103   yeats # --> your hostname here
192.168.1.198   yeatsHouse


# The following lines are desirable for IPv6 capable hosts
::1     ip6-localhost ip6-loopback
fe00::0 ip6-localnet
ff00::0 ip6-mcastprefix
ff02::1 ip6-allnodes
ff02::2 ip6-allrouters
`

---

## Hardware Setup

Proceedure for drone-Jetson communication is found [here](https://developer.dji.com/onboard-sdk/documentation/development-workflow/hardware-setup.html) note details on Advanced Sensing as they are required to use drone cameras.


## Debugging

Why it doesn't work?? 

### Vehicle Activation Errors

Check [here](https://forum.dji.com/thread-216529-1-1.html).