#include "ros/ros.h"
#include "std_msgs/Float32.h"
#include "position_sync/MissionWpSetSpeed.h"
#include "position_sync/MissionWpGetSpeed.h"
#include <dji_telemetry.hpp>
#include "geometry_msgs/Vector3Stamped.h"

/**
* This tutorial demonstrates simple receipt of messages over the ROS system.
*/
long double syncTarget = 0.0;
long double syncData = 0.0;
geometry_msgs::Vector3 currentVelocity;
ros::ServiceClient setSpeedClient;
position_sync::MissionWpSetSpeed setSpeedReq;
float_t currentSpeed;
float_t syncingSpeed = 1; // weighting of sync difference
float_t baseSpeedParam;

void velocityCallback(const geometry_msgs::Vector3Stamped::ConstPtr& msg)
// boost::shared_ptr<std_msgs::String>
{
  currentVelocity = msg->vector;
  currentSpeed = pow(pow(currentVelocity.x,2) + pow(currentVelocity.y,2) + pow(currentVelocity.z,2),0.5);
}

void updateSyncTargetCallback(const std_msgs::Float32::ConstPtr& msg)
{
  syncTarget = msg->data; // update sync signal we wish to match, from plath
}

void syncCallback(const std_msgs::Float32::ConstPtr& msg)
{
  syncData = msg->data;
  int waypointInt = (int)syncData;
  double waypointDec = syncData - waypointInt;

  // first only consider syncing when mid travel, starts and stops are weird
  if (waypointDec >= (float_t)0.05 && waypointDec <= (float_t)0.95)
  {
    if (abs(syncData - syncTarget) >= (float_t)0.05) // only sync if significantly different
    {
      // assuming a base velocity of 2 m/s
      DJI::OSDK::float32_t desiredSpeed = baseSpeedParam + syncingSpeed*(syncTarget - syncData);
      if (desiredSpeed <= 0.1)
      {
        desiredSpeed = 0.1; // min speed
      }
      setSpeedReq.request.speed = desiredSpeed;
      setSpeedClient.call(setSpeedReq);
      ROS_INFO("I set: %f", setSpeedReq.request.speed);
    }
  }
  else
  {
    // ROS_INFO("progress: %f", syncData);
  }


}

int main(int argc, char **argv)
{
  ros::init(argc, argv, "syncNode");
  ros::NodeHandle n;

  // load base speed value
  ros::param::get("base_speed", baseSpeedParam);

  ros::Subscriber syncSub2 = n.subscribe("sync_signal", 10, syncCallback);
  ros::Subscriber syncSub1 = n.subscribe("sync_target", 10, updateSyncTargetCallback);

  ros::Subscriber speedSub = n.subscribe("dji_osdk_ros/velocity", 10, velocityCallback);

  setSpeedClient = n.serviceClient<position_sync::MissionWpSetSpeed>(
    "dji_osdk_ros/mission_waypoint_setSpeed");

  ros::spin();
  return 0;
}
